import numpy as np
import pandas as pd
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt
import seaborn as sns
from sklearn.metrics import confusion_matrix

def vi_confusion_matrix(y_true, y_pred, filename, labels, ymap=None, figsize=(10,10)):
    """
    Generate matrix plot of confusion matrix with pretty annotations.
    The plot image is saved to disk.
    args: 
      y_true:    true label of the data, with shape (nsamples,)
      y_pred:    prediction of the data, with shape (nsamples,)
      filename:  filename of figure file to save
      labels:    string array, name the order of class labels in the confusion matrix.
                 use `clf.classes_` if using scikit-learn models.
                 with shape (nclass,).
      ymap:      dict: any -> string, length == nclass.
                 if not None, map the labels & ys to more understandable strings.
                 Caution: original y_true, y_pred and labels must align.
      figsize:   the size of the figure plotted.
    """
    if ymap != None:
        y_pred = [ymap[yi] for yi in y_pred]
        y_true = [ymap[yi] for yi in y_true]
        labels = [ymap[yi] for yi in labels]
    cm = confusion_matrix(y_true, y_pred, labels=labels)
    cm_sum = np.sum(cm, axis=1, keepdims=True)
    cm_perc = cm / cm_sum * 100
    annot = np.empty_like(cm).astype(str)
    nrows, ncols = cm.shape
    for i in range(nrows):
        for j in range(ncols):
            c = cm[i, j]
            p = cm_perc[i, j]
            if i == j:
                s = cm_sum[i]
                annot[i, j] = '%.1f%%\n%d/%d' % (p, c, s)
            elif c == 0:
                annot[i, j] = ''
            else:
                annot[i, j] = '%.1f%%\n%d' % (p, c)
    cm = pd.DataFrame(cm, index=labels, columns=labels)
    cm.index.name = 'Actual'
    cm.columns.name = 'Predicted'
    fig, ax = plt.subplots(figsize=figsize)
    sns.heatmap(cm, annot=annot, fmt='', ax=ax)
    plt.savefig(filename)

def vi_history(history, save_file_path):
    # Plot the loss and accuracy curves for training and validation 
    fig, ax = plt.subplots(2,1)
    ax[0].plot(history.history['loss'], color='b', label="Training loss")
    ax[0].plot(history.history['val_loss'], color='r', label="validation loss",axes =ax[0])
    legend = ax[0].legend(loc='best', shadow=True)

    ax[1].plot(history.history['acc'], color='b', label="Training accuracy")
    ax[1].plot(history.history['val_acc'], color='r',label="Validation accuracy")
    legend = ax[1].legend(loc='best', shadow=True)

    plt.savefig(save_file_path)


# -*- coding: utf-8 -*-
import os
import glob
import random
import shutil
import pickle
from datetime import datetime

# path operation
def create_file_path(dir_path, file_name):
    return os.path.join(dir_path, file_name)

def create_dir_path(dir_path, dir_name):
    return os.path.join(dir_path, dir_name)

def give_me_dir_path(parent_dir_path, pre_dir_name=""):
    dir_name = pre_dir_name + "_" + get_dateTime_str()
    return os.path.join(parent_dir_path, dir_name)

# diretory operation
def createDir (dirPath):
    if not os.path.exists(dirPath):
        os.makedirs(dirPath)

def removeDir (dirPath):
    if os.path.exists(dirPath):
        shutil.rmtree(dirPath, ignore_errors=True)

def renameDir (dirPath):
    if os.path.exists(dirPath):
        dt = datetime.now().strftime("%Y-%m-%d-%H-%M-%S")
        os.rename(dirPath, dirPath + "_bk_" + dt)

def get_dateTime_str():
    dt_str = datetime.now().strftime("%Y-%m-%d-%H-%M-%S")
    return dt_str

def remove_and_create_Dir(dirPath):
    removeDir(dirPath)
    createDir(dirPath)

def rename_and_create_Dir(dirPath):
    renameDir(dirPath)
    createDir(dirPath)

def get_parent_dirPath(filePath):
    return os.path.split(filePath)[0]

def get_parent_dirName(filePath):
    dirPath = os.path.split(filePath)[0]
    return os.path.split(dirPath)[1]

def get_dir_name(dirPath):
    return os.path.split(dirPath)[1]

def get_dir_name_list(dir_root_path):
    '''get dir_name list under root_dir
    '''
    dir_name_list = []
    for x in os.listdir(dir_root_path):
        xPath = dir_root_path + "/" + x
        if os.path.isdir(xPath):
            dir_name_list.append(x)
    return dir_name_list

def get_dir_path_list(dir_root_path):
    '''get dir_path list under root_dir
    '''
    dir_path_list = []
    for x in os.listdir(dir_root_path):
        xPath = dir_root_path + "/" + x
        if os.path.isdir(xPath):
            dir_path_list.append(xPath)
    return dir_path_list

# file operation
def copy_file(src_filePath, dst_filePath):
    if not os.path.exists(src_filePath):
        return False
    shutil.copyfile(src_filePath, dst_filePath)
    return True

def get_fileName_ext(filePath):
    return os.path.split(filePath)[1]

def get_fileName_no_ext(filePath):
    fileName_ext = os.path.basename(filePath)
    fileName_no_ext = os.path.splitext(fileName_ext)[0]
    return fileName_no_ext

def is_path_exists(path):
    return os.path.exists(path)

def get_all_files_pathList(dirPath, ext):
    file_pathList = []
    for path, subdirs, files in os.walk(dirPath):
        for name in files:
            if name.endswith('.' + ext) and not name.startswith('.'):
                filePath = os.path.join(path, name)
                file_pathList.append(filePath)
    return file_pathList

def get_file_name_list(dirPath, ext):
    file_name_list = []
    for x in os.listdir(dirPath):
        if x.endswith('.' + ext) and not x.startswith('.'):
            file_name_list.append(x)
    return file_name_list

def get_file_path_list(dirPath, ext):
    file_path_list = []
    for x in os.listdir(dirPath):
        if x.endswith('.' + ext) and not x.startswith('.'):
            file_path_list.append(dirPath + "/" + x)
    return file_path_list

def count_files(dirpath, ext):
    return len(get_file_name_list(dirpath, ext))

#want to remove functions ####################################################################
def getSubDirListOf(dirPath):
    dirList = []
    for x in os.listdir(dirPath):
        xPath = dirPath + "/" + x
        """
        if not os.path.isabs(xPath):
            xPath = os.path.dirname(os.path.abspath(__file__)) + "/" + xPath
        """
        if os.path.isdir(xPath):
            dirList.append(x)
        """
        else:
            print(x + " is not dir")
        """
    return dirList

def getFileListOf(dirPath, ext):
    fileList = []
    for x in os.listdir(dirPath):
        if x.endswith('.' + ext) and not x.startswith('.'):
            fileList.append(x)
    return fileList

def getPathListOf(dirPath, ext):
    fileList = []
    for x in os.listdir(dirPath):
        if x.endswith('.' + ext) and not x.startswith('.'):
            fileList.append(dirPath + "/" + x)
    return fileList

def countFilesOf(rootDirPath, ext):
    dirList = getSubDirListOf(rootDirPath)
    count = 0
    for i in range(len(dirList)):
        if not dirList[i].startswith('.'):
            dirPath = rootDirPath + "/" + dirList[i]
            # get files
            fileList = getFileListOf(dirPath, ext)
            count += len(fileList)
    return count

def exportStrList2File(strList, otFilePath):
    '''export string list to file
    '''
    f = open(otFilePath, "w")
    for line in strList:
        line = line + "\n"
        try:
            f.write(line.encode('utf_8'))
        except Exception as ex:
            print(ex)
    f.close()

def loadLineListFromFile(filePath):
    f = open(filePath)
    lines = f.readlines()
    f.close()
    lineList = []
    for line in lines:
        line = line.replace('\n', '')
        line = line.replace('\r', '')
        lineList.append(line)
    return lineList

def dumpStrList2File(strList, otFilePath):
    with open(otFilePath, 'wb') as f:
        pickle.dump(strList, f)

def loadFile2StrList(itFilePath):
    with open(itFilePath, 'rb') as f:
        strList = pickle.load(f)
    return strList


from setuptools import setup

requires = []

setup(
    name='skz_utils',
    version='0.1.2',
    description='python utils',
    url='https://github.com/skyzhao3q/skyz_pyutils.git',
    author='zhao-tian',
    author_email='skyzhaoi3q@gmail.com',
    license='MIT',
    keywords='some python utils',
    packages=[
        "skz_utils",
    ],
    install_requires=requires,
    classifiers=[
        'Programming Language :: Python :: 3.0',
    ],
)


from . import os_utils
from . import im_utils
import cv2
import os
import numpy as np

def read_img_file(img_file_path, input_shape, threshold_val=None):
    '''read image and convert to input_shape
    '''
    w, h, c = input_shape
    if c == 3:
        mode = 'rgb'
    if c == 1:
        mode = 'gray'
        if threshold_val is not None:
            mode = 'bin'
    img = im_utils.read(img_file_path, mode=mode, threshold_val=threshold_val)
    # resize
    img = cv2.resize(img, (w, h))
    # expand shape
    img = np.expand_dims(img, axis=0)

    return img

def read_img_dir(img_dir_path, ext, input_shape, threshold_val):
    '''read image directory and convert to input_shape
    '''
    img_list = []
    img_file_path_list = os_utils.get_all_files_pathList(img_dir_path, ext)
    for img_file_path in img_file_path_list:
        img = read_img_file(img_file_path, input_shape, threshold_val=threshold_val)
        img_list.append(img)
    # reshape
    img_ds = np.array(img_list).reshape(-1, input_shape[0], input_shape[1], input_shape[2])

    return img_ds, img_file_path_list

def read_img_label_dir(img_label_dir_path, ext, input_shape, threshold_val):
    ''' read image-label directory
    '''
    X, img_file_path_list = read_img_dir(img_label_dir_path, ext, input_shape, threshold_val)
    y = []
    for img_file_path in img_file_path_list:
        label = os_utils.get_dir_name(img_file_path)
        try:
            label = int(label)
        except ValueError:
            assert False, "<ds_utils: read_img_label_dir> Invalid Label Name: {}".format(label)
        y.append(label)
    
    return X, y, img_file_path_list

def image_dir_2array(img_dir_path, ext, img_format='gray', th_value=None):
    '''create image array
    '''
    img_list = []
    name_list = []
    # read image
    for img_path in os_utils.get_all_files_pathList(img_dir_path, ext):
        img_name = os_utils.get_fileName_ext(img_path)
        name_list.append(img_name)
        # read image data
        if img_format == 'gray':
            img = cv2.imread(img_path, 0)
            if th_value is not None:
                _,img = cv2.threshold(img,th_value,255,cv2.THRESH_BINARY)
        elif img_format == 'rgb':
            img = cv2.imread(img_path, 1)
            img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
        else:
            assert False, "Invalid Image Format! {}".format(img_format)
        # create record
        img_list.append(img)

    return name_list, img_list

def image_array_from_dir(img_dir_path, input_shape, ext, th_value=None):
    '''create image array
    '''
    img_W, img_H, img_C = input_shape
    x_list = []
    # read image
    for img_path in os_utils.get_all_files_pathList(img_dir_path, ext):
        print(img_path)
        # read image data
        if img_C == 1:
            img = cv2.imread(img_path, 0)
            if th_value is not None:
                _,img = cv2.threshold(img,th_value,255,cv2.THRESH_BINARY)
        elif img_C == 3:
            img = cv2.imread(img_path, 1)
            img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
        else:
            assert False, "<create_dataset> Invalid img_C value! {}".format(img_C)
        img = cv2.resize(img, (img_W, img_H))
        # create record
        x_list.append(img)
    # dataset
    X = np.array(x_list).reshape(-1,img_W, img_H, img_C)

    return X

def image_dataset_from_dir(img_dir_path, input_shape, ext, th_value = None):
    '''create image dataset with label from directory
    '''
    img_W, img_H, img_C = input_shape
    y_list = []
    x_list = []
    # read category
    for dir_name in os_utils.get_dir_name_list(img_dir_path):
        dir_path = os.path.join(img_dir_path, dir_name)
        # read image
        for img_path in os_utils.get_file_path_list(dir_path, ext):
            print(img_path)
            # read image data
            if img_C == 1:
                img = cv2.imread(img_path, 0)
                if th_value is not None:
                    _,img = cv2.threshold(img,th_value,255,cv2.THRESH_BINARY)
            elif img_C == 3:
                img = cv2.imread(img_path, 1)
                img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
            else:
                assert False, "<create_dataset> Invalid img_C value! {}".format(img_C)
            img = cv2.resize(img, (img_W, img_H))
            # create record
            y_list.append(int(dir_name))
            x_list.append(img)
    # dataset
    y = np.array(y_list)
    X = np.array(x_list).reshape(-1,img_W, img_H, img_C)

    return y, X


import imutils
import cv2
import matplotlib.pyplot as plt

def read(img_file_path, mode=('rgb', 'gray', 'bin'), threshold_val=None):
    ''' read image
    '''
    if mode not in ('rgb', 'gray', 'bin'):
        assert False, "<im_utils> Invalid Mode: {}".format(mode)
    if mode == 'rgb':
        img = cv2.imread(img_file_path, 1)
        img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
    if mode == 'gray':
        img = cv2.imread(img_file_path, 0)
    if mode == 'bin':
        if threshold_val is None:
            assert False, "<im_utils> NO threshold value"
        img = cv2.imread(img_file_path, 0)
        _,img = cv2.threshold(img,threshold_val,255,cv2.THRESH_BINARY)
    
    return img

def write(img_file_path, img):
    _,_,c = img.shape
    if c == 3:
        img = cv2.cvtColor(img, cv2.COLOR_RGB2BGR)
    cv2.imwrite(img_file_path, img)

def rotate(img, angle):
    img = imutils.rotate(img, angle=angle)

    return img

def rotate_bound(img, angle):
    '''rotate without cut off image
       fill with black pixel
    '''
    img = imutils.rotate_bound(img, angle=angle)

    return img

def rotate_bound_white(img, angle):
    '''rotate without cut off image
       fill with white pixel
    '''
    # convert white 2 black
    img = ~img
    # fill with black
    img = imutils.rotate_bound(img, angle=angle)
    # recover 2 white
    img = ~img

    return img

def resize_with_aspect(img, width=None, height=None):
    img = imutils.resize(img, width=width, height=height)

    return img

def bgr2rgb(img):
    img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)

    return img

def show_img(img, title=None):
    if title is not None:
        plt.figure(title)
    plt.imshow(img)
    plt.show()

def show_gray_img(img, title=None, vmin=0, vmax=255):
    if title is not None:
        plt.figure(title)
    plt.imshow(img, cmap='gray', vmin=vmin, vmax=vmax)
    plt.show()

